@@ -21,6 +21,7 @@ from Products.PageTemplates.PageTemplateFile import PageTemplateFile
 from Products.CMFCore.interfaces import ISiteRoot
 from ilrt.migrationtool.browser.interfaces import ISiteMigrationTool
 from ilrt.migrationtool.browser.workflowtool import WorkflowMigrationView
+from ilrt.migrationtool.browser.atfieldtool import ATFieldIndexInfo
 from ilrt.migrationtool.browser import utils
 
 PRE_MIGRATION_RELEASE = "0.1"
@@ -59,13 +60,23 @@ class SiteMigrationTool(BaseMTool):
         return self.options
     
     def __init__(self):
-        self.migrations = []
+        """ Initialization just calls the reset of attributes """
+        self.wfmt = WorkflowMigrationView(self,getattr(self,'REQUEST',{}))
+        self.atfit = ATFieldIndexInfo()  
+        self.clearMigrations()
+
+    def clearMigrations(self):
+        ''' Deletes stored migration objects and upgrade / downgrade methods '''
+        self.migrations = {}
+        self.methodlists = {}        
         self._upgradePaths = {}
         self._downgradePaths = {}
         self.migration_id = ''
         self.egg = ''
-  
+        return
+        
     def log(self,message,summary='',severity=0):
+        """ Logs errors """
         zLOG.LOG('Site (MigrationTool): ', severity, summary, message)
 
     def _logErrorOut(self, out, handle='raise'):
@@ -83,34 +94,62 @@ class SiteMigrationTool(BaseMTool):
     def setMigration(self, migration_id):
         """ Set the migration object whose method
             is to be to run individually """
-        self.migration_id = migration_id
+        request = getattr(self,'REQUEST',{})
+        if request.get('migration_id',''):
+            self.migration_id = migration_id
+            self.getMigrationMethods()
+            request.RESPONSE.redirect('@@manage_run_migration_method')
+        return
 
     def getMigrationMethods(self):
         """ List all the methods for a particular migration object """
-        methods = []
         if not self.migration_id:
-            return methods
+            return []
         else:
-            for m in self.migrations:
-                if m.migration == self.migration_id:
-                    attribs = [me for me in dir(m) if not me.startswith('__')]
-                    for attrib in attribs:
-                        # if type(attrib) != StringType:
-                        methods.append(attrib)
-            return methods
+            if self.methodlists:
+                return self.methodlists.get(self.migration_id,[])
 
-    def runMigrationMethod(self,method):
-        """ Run an individual method from a migration object """
+    def runMigrationMethod(self,method,**kwargs):
+        """ Run an individual method from a migration script
+            Note that whether a migration method can be run individually or
+            even whether individual methods are used is down to the author
+            of the migration scripts
+            This just provides them with an extra debugging tool
+        """
         portal = getUtility(ISiteRoot)
-        for m in self.migrations:
-            if m.migration == self.migration_id:
-                me = getattr(m,method,None)
-                if me:
-                    try:
-                        out = str(me(portal))
-                    except:
-                        out = str(me) + str(Error)
+        out = ["Ran individual migration method %s" % method,]
+        if self.methodlists.has_key(self.migration_id):
+            if method in self.methodlists.get(self.migration_id,[]):
+                try:
+                    exec('from ' + self.egg + ' import migrations')
+                except ImportError:
+                    out.append("Error: Sorry this egg, " + str(self.egg) + ", has no migrations \
+                        - please add a migrations folder with migration scripts in")
                     return out
+                except:
+                    return self.traceout(out)
+                mig = getattr(migrations,self.migration_id, None)
+                if mig:
+                    me = getattr(mig,method,None)
+                    if me:
+                        try:
+                            if method in ('upgrade','downgrade'):
+                                out.extend(me(self.aq_parent))
+                            else:
+                                me(self.aq_parent, out)
+                        except:
+                            out = self.traceout(out,'Error from running %s' % method)
+                    else:
+                        out.append('Sorry the method could not be extracted from the migration script')
+                else:
+                    out.append('Sorry the migration script could not be extracted from the migrations directory')
+            else:
+                out.append('Sorry the method %s was not found in %s' % (method, self.migration_id))                
+        else:
+            out.append('Sorry the methods for %s were not found' % self.migration_id)
+
+        out = [(msg, '') for msg in out]
+        return self.manage_results(self,out=out)
 
     def registerUpgradePath(self, oldversion, newversion, function):
         """ Basic register func """
@@ -138,6 +177,7 @@ class SiteMigrationTool(BaseMTool):
             At this point look up installed instance version
             if its in quickinstaller
         '''
+        self.clearMigrations()
         if context_id:
             self.egg = str(context_id)
         if self.egg:
@@ -150,13 +190,29 @@ class SiteMigrationTool(BaseMTool):
             out.extend(self.loadMigrations(reload=1))
         return self.manage_overview(self, out=out)
 
+    def traceout(self, out,
+           msg='Your migration scripts caused a fatal error please correct it'):
+        ''' Push error traceback out to ZMI interface '''
+        out.append('Error: %s' % msg)
+        out.append('TRACEBACK ...')
+        errlines = traceback.format_exc().split('File')
+        if len(errlines)>3:
+            errlines = errlines[3:]
+        for err in errlines:
+            out.append(err)
+        return out
+
+    def importMigrations(self, out):
+        ''' Import the migrations in the source egg to the tool '''
+
     def loadMigrations(self, reload=0):
         ''' Search for sitemigrations in the specified egg '''
-        out = [] 
+        out = []
         if not self.egg:
             return out
         if reload or not getattr(self,'migrations',None):
-            self.migrations = []
+            self.migrations = {}
+            self.methodlists = {}
             try:
                 exec('from ' + self.egg + ' import migrations')
             except ImportError:
@@ -164,42 +220,41 @@ class SiteMigrationTool(BaseMTool):
                     - please add a migrations folder with migration scripts in")
                 return out
             except:
-                errlines = traceback.format_exc().split('File')
-                out.append('Error: Your migration scripts caused a fatal error please correct it')
-                out.append('TRACEBACK ...')
-                if len(errlines)>3:
-                    errlines = errlines[3:]
-                for err in errlines:
-                    out.append(err)
-                return out
-            migrationlist = [m for m in dir(migrations) \
-                                if not m.startswith('__')]
+                return self.traceout(out)
+            try:
+                migrationlist = [m for m in dir(migrations) \
+                                            if not m.startswith('__')]
+            except:
+                migrationlist = []
             if not migrationlist:
                 out.append("Error: the migrations folder has no initialized \
                             migrations - check you have added them in the init")  
             util_methods = dir(utils)
             for m in migrationlist:
-                migration = getattr(migrations,m)
-                methodlist = [me for me in dir(migration) \
+                m = str(m)
+                try:
+                    migration = getattr(migrations,m)
+                    methodlist = [me for me in dir(migration) \
                                  if not me.startswith('__')]
-                methodlist = list(set(methodlist) - set(util_methods))
+                    methodlist = list(set(methodlist) - set(util_methods))
+                except:
+                    methodlist = []
+                    msg = 'Failed to get method list from migration script %s please check your methods' % m
+                    out = self.traceout(out, msg)
                 if 'VERSIONS' in methodlist:
                     if 'upgrade' in methodlist:
-                        mig = createObject(u"SiteMigration")
-                        mig.migration = m
-                        mig.versions = migration.VERSIONS
-                        methods = {}
-                        # Unpicklable objects could be returned which we skip
-                        for me in methodlist:
-                            try:
-                                methods[me] = getattr(migration,me,None)
-                            except:
-                                pass
                         try:
-                            mig.methods(**methods)
+                            self.registerUpgradePath(migration.VERSIONS[0],
+                                                     migration.VERSIONS[1],
+                                                     migration.upgrade)
+                            if hasattr(migration,'downgrade'):
+                                self.registerDowngradePath(migration.VERSIONS[1],
+                                                           migration.VERSIONS[0],
+                                                           migration.downgrade)
+                            self.migrations[m] = migration.VERSIONS
+                            self.methodlists[m] = [me for me in methodlist if me != me.upper()]
                         except:
-                            pass
-                        out.append('Added migration ' + m)
+                            out = self.traceout(out,'Failed to register migration %s' % m)
                     else:
                         out.append("Error: Sorry migration script '" + m + \
                                    "' has no upgrade function please add \
@@ -208,14 +263,6 @@ class SiteMigrationTool(BaseMTool):
                     out.append("Error: Sorry migration script '" + m + \
                                    "' has no VERSIONS please add one \
                                     ie. VERSIONS = ('1','2')")
-        if self.migrations:
-            for migration in self.migrations:
-                self.registerUpgradePath(migration.versions[0],
-                                         migration.versions[1],
-                                         migration.upgrade)
-                self.registerDowngradePath(migration.versions[1],
-                                           migration.versions[0],
-                                           migration.downgrade)
         else:
             out.append('Error: Sorry no migrations found')
         return out
@@ -315,11 +362,9 @@ class SiteMigrationTool(BaseMTool):
         
         return self._migrate(version, "up")
 
-
-    security.declareProtected(ManagePortal, 'runWorkflowMigration')
+    security.declareProtected(ManagePortal, 'runWorkflowTransfer')
     def runWorkflowTransfer(self, out, wf_from, wf_to, mapping, container=None):
-        wfmt = WorkflowMigrationView(self,getattr(self,'REQUEST',{}))
-        out.append(wfmt._runTransfer(wf_from, wf_to, mapping, container))
+        out.append(self.wfmt._runTransfer(wf_from, wf_to, mapping, container))
         return
 
 InitializeClass(SiteMigrationTool)
