@@ -7,26 +7,34 @@ SAVEPATH=os.path.join(INSTANCE_HOME, 'var', 'site_migration')
 def runGenericSetupSteps(portal, out,
                          profile = 'mysite.theme:default',steps=[]):
     """Utility method - Remember you need to specify the right profile
-       ... or else you may wipe everything back to base """
+       ... or else you may wipe everything back to base
+       
+       When using SelectedSteps you need the name of the class in
+       importexport - NOT the tool name - e.g. skins, atcttool, catalog etc.
+       see portal_setup/manage_importSteps checkboxes for the ids of tools
+    """
     gsetup = getToolByName(portal, 'portal_setup')
-    context_id = 'profile-%s' % profile 
+    mtool = getToolByName(portal, 'site_migration')
+    context_id = 'profile-%s' % profile
+    ssout = []
     if not steps:
         try:
             gsetup.manage_importAllSteps(context_id=context_id)
         except ValueError:
-            out.append("Sorry you submitted an invalid value " \
-                       + str(ValueError))
+            ssout = mtool.traceout(ssout,
+                                  "Generic setup steps failed for %s" % profile)
     else:
         try:
             gsetup.manage_importSelectedSteps(context_id=context_id,
                                               ids=steps,
                                               run_dependencies=True)
         except ValueError:
-            out.append("Sorry you submitted an invalid value " \
-                       + str(ValueError))
-
-    out.append("Finished doing generic setup steps for " + profile)
-
+            ssout = mtool.traceout(ssout,
+                                  "Generic setup steps failed for %s" % profile)
+    if ssout:
+        out.extend(ssout)
+    else:
+        out.append("Finished doing generic setup steps for " + profile)
     return out
 
 def installProducts(portal, out, products = []):
@@ -55,6 +63,28 @@ def removeProducts(portal,out, products = []):
         out.append('No products uninstalled')
     return out
 
+
+def editSiteProps(portal, out, sheet='site_properties',
+                  prop='enable_link_integrity_checks',value=False):
+    """ Utility function to (temporarily) switch portal properties -
+        permanent changes can be done via generic setup
+        Default use case is to switch off link checks whilst replacing content
+    """
+    portal_props = getToolByName(portal, 'portal_properties')
+    propsheet = portal_props.get(sheet, None)
+    if propsheet:
+        if hasattr(propsheet,prop):
+            try:
+                propsheet._updateProperty(prop,value)
+                out.append('Set %s %s to %s' % (sheet,prop,propsheet.getProperty(prop)))
+            except:
+                out.append('Property %s cannot be %s' % (prop,value))
+        else:
+            out.append('Failed to find %s property %s' % (sheet,prop))
+    else:
+        out.append('Failed to find property sheet %s' % sheet)
+    return out
+
 def pickleAttributes(portal,out,attributes,path,action='save'):
     """ This function is for saving persistent attributes from objects
         to file, or reloading them, where they are not handled by generic
@@ -141,3 +171,5 @@ def pickleAttributes(portal,out,attributes,path,action='save'):
         else:
             out.append('The object %s has none of the listed attributes' \
                                                              % obj.getId())
+
+
